/// <reference types="node" />
import type { IBinaryData, INodeExecutionData } from 'n8n-workflow';
import type { BinaryMetadata, IBinaryDataConfig } from '../Interfaces';
export declare class BinaryDataManager {
    static instance: BinaryDataManager | undefined;
    private managers;
    private binaryDataMode;
    private availableModes;
    constructor(config: IBinaryDataConfig);
    static init(config: IBinaryDataConfig, mainManager?: boolean): Promise<void>;
    static getInstance(): BinaryDataManager;
    copyBinaryFile(binaryData: IBinaryData, filePath: string, executionId: string): Promise<IBinaryData>;
    storeBinaryData(binaryData: IBinaryData, binaryBuffer: Buffer, executionId: string): Promise<IBinaryData>;
    retrieveBinaryData(binaryData: IBinaryData): Promise<Buffer>;
    retrieveBinaryDataByIdentifier(identifier: string): Promise<Buffer>;
    getBinaryPath(identifier: string): string;
    getBinaryMetadata(identifier: string): Promise<BinaryMetadata>;
    markDataForDeletionByExecutionId(executionId: string): Promise<void>;
    markDataForDeletionByExecutionIds(executionIds: string[]): Promise<void>;
    persistBinaryDataForExecutionId(executionId: string): Promise<void>;
    deleteBinaryDataByExecutionId(executionId: string): Promise<void>;
    duplicateBinaryData(inputData: Array<INodeExecutionData[] | null> | unknown, executionId: string): Promise<INodeExecutionData[][]>;
    private generateBinaryId;
    private splitBinaryModeFileId;
    private duplicateBinaryDataInExecData;
}
